/* 
 * FUNCIONALIDADE: Insere permissões de acesso a relatórios a partir dos grupos de relatórios configurados no ICoruja 
 * 1 - Busta todos os grupos de relatórios cadastrados no ICoruja
 * 2 - Com os grupos de relatórios, busca todos os relatórios pertencentes a esses grupos
 * 3 - Une os relatórios com os id dos módulos onde dos grupos de relatórios estão configurados
 * 4 - Une com o id do perfis de usuário que possuem permissão de acesso ao módulo
 * 5 - Verificar se há opção de menu com permissão para o relatório/modulo/perfil
 * 6 - Caso não haja, verifica se há opção de menu com acesso negado
 * 6.1 - Se sim, altera permissão de acesso a opção de menu para o perfil
 * 6.2 - Se não, verifica se existe opção de menu cadastrada
 * 6.2.1 - Se sim, cria permissão a opção de menu
 * 6.2.2 - Se não, cria opção de menu
 * 6.2.2.1 - Cria permissão a opção de menu 
 * PRÉ-REQUISITOS:
 * Necessita dar permissão de acesso a tabelas de outras bases para o usuário da base do ICoruja
 *
 * -- Executar os scripts abaixo na conexão com o banco do Security:
 * GRANT ALL ON TB_PERFIL_MODULO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_MODULO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_OPCAO_MENU_PERMISSAO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_OPCAO_MENU TO BANCO_ICORUJA;
 * GRANT ALL ON OME_ID TO BANCO_ICORUJA;
 * GRANT ALL ON OMP_ID TO BANCO_ICORUJA;
 * -- Executar os scripts abaixo na conexão com o banco do Report:
 * GRANT ALL ON TB_REL_RELATORIO TO BANCO_ICORUJA;
 *
 * DATE: 27/04/2023
 * AUTOR: andre.zappe
 * PARAMETROS: Sem parâmetros
 * EX: BEGIN PROC_ADD_PERMISSAO_RELATORIOS; END;

*/ 

create or replace PROCEDURE PROC_ADD_PERMISSAO_RELATORIOS AS
    v_RELATORIO_CODTEL          VARCHAR2(255);
    v_RELATORIO_NOME            VARCHAR2(255);
    v_MODULO_ID                 NUMBER(10, 0);
    v_PERFIL_ID                 NUMBER(10, 0);
    v_COUNT_PERMISSAO_RELATORIO NUMBER(10, 0) := 0;
    v_PERMISSAO_MENU_NEGADO_ID  NUMBER(10, 0);
    v_OPCAO_MENU_ID             NUMBER(10, 0);
    I                           NUMBER(10, 0) := 0;
    CURSOR CURSOR_RELATORIOS IS -- BUSCA RELATÓRIOS, MÓDULOS E PERFIS COM ACESSO AO MÓDULO
        SELECT DISTINCT REL_CODTEL, REL_NOME, MOD_ID, PFM_PUSID
        FROM (
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT RGM_GRURELREMAN AS GRUPO, MOD_ID FROM TB_REGRA_MATRICULA, EDUSOFT_SECURITY.TB_MODULO WHERE RGM_GRURELREMAN IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELCER AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.TB_MODULO WHERE PSU_GRURELCER IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') TABELA WHERE REL_GRUID IN (TABELA.GRUPO) AND ROWNUM = 1
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELMLO AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.TB_MODULO WHERE PSU_GRURELMLO IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELHIS AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.TB_MODULO WHERE PSU_GRURELHIS IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELDOS AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELDOS IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT CPA_GRURELCTBL AS GRUPO, MOD_ID FROM TB_CONT_PARAMETROS, EDUSOFT_SECURITY.TB_MODULO WHERE CPA_GRURELCTBL IS NOT NULL AND MOD_TIPOMOD = 'CONTABIL') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PFI_GRURELDIVATIID AS GRUPO, MOD_ID FROM TB_PARAMETROS_FINANCEIRO, EDUSOFT_SECURITY.TB_MODULO WHERE PFI_GRURELDIVATIID IS NOT NULL AND MOD_TIPOMOD = 'FINANCEIRO') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCPG AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCPG IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELREN AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELREN IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCAI AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCAI IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCPGCARTAO AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCPGCARTAO IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELAVA AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELAVA IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELINS AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELINS IS NOT NULL) TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.TB_REL_RELATORIO, (SELECT DISTINCT PRE_GRUREL AS GRUPO, MOD_ID FROM TB_PARAMETROS_REQUERIMENTO, EDUSOFT_SECURITY.TB_MODULO WHERE PRE_GRUREL IS NOT NULL AND MOD_TIPOMOD = 'REQUERIMEN') TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             ) TABELA,
             EDUSOFT_SECURITY.TB_PERFIL_MODULO
        WHERE PFM_MODID = MOD_ID
          AND PFM_PUSID IS NOT NULL;
BEGIN

    dbms_output.put_line('[START] PROC_ADD_PERMISSAO_RELATORIOS ');

    OPEN CURSOR_RELATORIOS;
    FETCH CURSOR_RELATORIOS INTO v_RELATORIO_CODTEL, v_RELATORIO_NOME, v_MODULO_ID, v_PERFIL_ID;
    WHILE CURSOR_RELATORIOS%FOUND
        LOOP
            BEGIN
                I := I + 1;
                dbms_output.put_line('>>>>v_RELATORIO_CODTEL - ' || v_RELATORIO_CODTEL ||
                                     ' | v_RELATORIO_NOME - ' || v_RELATORIO_NOME ||
                                     ' | v_MODULO_ID - ' || TO_CHAR(v_MODULO_ID) ||
                                     ' | v_PERFIL_ID - ' || TO_CHAR(v_PERFIL_ID));

                -- BUSCA SE HÁ ALGUMA PERMISSÃO PARA O RELATÓRIO, NO MÓDULO PARA O PERFIL DE USUÁRIO
                BEGIN
                    SELECT COUNT(OME_ID)
                    INTO v_COUNT_PERMISSAO_RELATORIO
                    FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                             INNER JOIN EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                                        ON (OME_ID = OMP_OMEID AND OMP_PUSID = v_PERFIL_ID)
                    WHERE (OME_ACTION = v_RELATORIO_CODTEL OR OME_CODRELATORIO = v_RELATORIO_CODTEL)
                      AND OME_TIPO IN ('RELATORIO', 'OPCAONOR')
                      AND OME_MODID = v_MODULO_ID
                      AND OMP_TIPPER != 'TPA1';
                EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                        v_COUNT_PERMISSAO_RELATORIO := 0;
                END;

                dbms_output.put_line('-----v_COUNT_PERMISSAO_RELATORIO - ' || TO_CHAR(v_COUNT_PERMISSAO_RELATORIO));

                IF v_COUNT_PERMISSAO_RELATORIO = 0 THEN -- CASO NÃO HAJA PERMISSÃO
                -- BUSCA SE HÁ ALGUMA PERMISSÃO NEGANDO O ACESSO AO MENU
                    BEGIN
                        SELECT OMP_ID
                        INTO v_PERMISSAO_MENU_NEGADO_ID
                        FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                 INNER JOIN EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                                            ON (OME_ID = OMP_OMEID AND OMP_PUSID = v_PERFIL_ID)
                        WHERE (OME_ACTION = v_RELATORIO_CODTEL OR OME_CODRELATORIO = v_RELATORIO_CODTEL)
                          AND OME_TIPO = 'RELATORIO'
                          AND OME_MODID = v_MODULO_ID
                          AND OMP_TIPPER = 'TPA1'
                          AND ROWNUM = 1;
                    EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                            v_PERMISSAO_MENU_NEGADO_ID := 0;
                    END;

                    dbms_output.put_line('-----v_PERMISSAO_MENU_NEGADO_ID - ' ||
                                         TO_CHAR(COALESCE(v_PERMISSAO_MENU_NEGADO_ID, 0)));

                    IF v_PERMISSAO_MENU_NEGADO_ID != 0 THEN-- HÁ PERMISSÃO CADASTRADA CONTUDO ESTÁ NEGADA, ALTERARÁ PARA POSSUIR PERMISSÃO NA OPÇÃO DE MENU
                        dbms_output.put_line('-----UPDATE TB_OPCAO_MENU_PERMISSAO');

                        UPDATE EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                        SET OMP_TIPPER            = 'TPA5',
                            DATA_ULT_ALTERACAO    = current_timestamp,
                            USUARIO_ULT_ALTERACAO = 'EDUSOFT_PERMISSAO_RELATORIO',
                            VERSAO                = VERSAO + 1
                        WHERE OMP_ID = v_PERMISSAO_MENU_NEGADO_ID;

                        COMMIT;

                    ELSE -- SE NÃO HÁ PERMISSÃO CADASTRADA, VERIFICA/CRIA A OPÇÃO DE MENU PARA O RELATÓRIO E CRIA A PERMISSÃO PARA O PERFIL DE USUÁRIO
                        BEGIN
--                             VERIFICA SE HÁ OPÇÃO DE MENU CRIADA
                            BEGIN
                                SELECT OME_ID
                                INTO v_OPCAO_MENU_ID
                                FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                WHERE (OME_ACTION = v_RELATORIO_CODTEL OR OME_CODRELATORIO = v_RELATORIO_CODTEL)
                                  AND OME_TIPO = 'RELATORIO'
                                  AND OME_MODID = v_MODULO_ID
                                  AND ROWNUM = 1;
                            EXCEPTION
                                WHEN NO_DATA_FOUND THEN
                                    v_OPCAO_MENU_ID := 0;
                            END;

                            dbms_output.put_line('-----v_OPCAO_MENU_ID - ' || TO_CHAR(COALESCE(v_OPCAO_MENU_ID, 0)));

                            IF v_OPCAO_MENU_ID = 0 THEN-- CRIA A OPÇÃO DE MENU PARA O RELATÓRIO
                                dbms_output.put_line('-----INSERT TB_OPCAO_MENU');

                                INSERT INTO EDUSOFT_SECURITY.TB_OPCAO_MENU (OME_ID, OME_DESCRICAO, OME_ACTION, OME_MODID,
                                                                                OME_ORDEM, OME_TIPO, VERSAO,
                                                                                DATA_ULT_ALTERACAO,
                                                                                USUARIO_ULT_ALTERACAO, OME_TIPACEITAPER,
                                                                                OME_TIPOMENU, OME_TIPOACAOMENU,
                                                                                OME_CODRELATORIO, OME_PARAMETROS,
                                                                                OME_ICONE, OME_EXIMOB,
                                                                                OME_APPEXIBEPAGPRI)
                                VALUES (EDUSOFT_SECURITY.OME_ID.NEXTVAL, v_RELATORIO_NOME, v_RELATORIO_CODTEL, v_MODULO_ID, 0, 'RELATORIO', 0,
                                        current_timestamp, 'EDUSOFT_PERMISSAO_RELATORIO', 'TAP1', 'RELATORIO', 'NORMAL',
                                        v_RELATORIO_CODTEL, '', '', 0, 0);

                                COMMIT;

                                SELECT OME_ID
                                INTO v_OPCAO_MENU_ID
                                FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                WHERE (OME_ACTION = v_RELATORIO_CODTEL OR OME_CODRELATORIO = v_RELATORIO_CODTEL)
                                  AND OME_TIPO = 'RELATORIO'
                                  AND OME_MODID = v_MODULO_ID
                                  AND ROWNUM = 1;

                                dbms_output.put_line('-----v_OPCAO_MENU_ID - ' || TO_CHAR(COALESCE(v_OPCAO_MENU_ID, 0)));
                            END IF;

                            dbms_output.put_line('-----[START] INSERT TB_OPCAO_MENU_PERMISSAO');

                            -- CRIA PERMISSÃO PARA O MENU
                            INSERT INTO EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO (OMP_ID, OMP_PUSID, OMP_OMEID, OMP_TIPPER, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO)
                             VALUES (EDUSOFT_SECURITY.OMP_ID.NEXTVAL, v_PERFIL_ID , v_OPCAO_MENU_ID, 'TPA5', 0, current_timestamp, 'EDUSOFT_PERMISSAO_RELATORIO');

                            COMMIT;
                        END;
                    END IF;
                END IF;

                FETCH CURSOR_RELATORIOS INTO v_RELATORIO_CODTEL, v_RELATORIO_NOME, v_MODULO_ID, v_PERFIL_ID;
            END;
        END LOOP;
    CLOSE CURSOR_RELATORIOS;

    dbms_output.put_line('[END] PROC_ADD_PERMISSAO_RELATORIOS');
END;
