/*
 * FUNCIONALIDADE: Insere permissões de acesso a interfaces a partir dos módulos cadastrados
 * 1 - Busca todos as interfaces cadastrados no ICoruja
 * 2 - Une as interfaces com os id dos módulos onde a interface está configurada
 * 3 - Une com o id do perfis de usuário que possuem permissão de acesso ao módulo
 * 4 - Verificar se há opção de menu com permissão para o interface/modulo/perfil
 * 5 - Caso não haja, verifica se há opção de menu com acesso negado
 * 5.1 - Se sim, altera permissão de acesso a opção de menu para o perfil
 * 5.2 - Se não, verifica se existe opção de menu cadastrada
 * 5.2.1 - Se sim, cria permissão a opção de menu
 * 5.2.2 - Se não, cria opção de menu
 * 5.2.2.1 - Cria permissão a opção de menu


 * Necessita dar permissão de acesso a tabelas de outras bases para o usuário da base do ICoruja
 *
 * -- Executar os scripts abaixo na conexão com o banco do Security:
 * GRANT ALL ON TB_PERFIL_MODULO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_MODULO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_OPCAO_MENU_PERMISSAO TO BANCO_ICORUJA;
 * GRANT ALL ON TB_OPCAO_MENU TO BANCO_ICORUJA;
 * GRANT ALL ON OME_ID TO BANCO_ICORUJA;
 * GRANT ALL ON OMP_ID TO BANCO_ICORUJA;
 *
 * DATE: 15/05/2023
 * AUTOR: andre.zappe
 * PARAMETROS: Sem parâmetros
 * EX: BEGIN PROC_ADD_PERMISSAO_INTERFACE; END;

*/

create or replace PROCEDURE PROC_ADD_PERMISSAO_INTERFACE AS
    v_INTERFACE_CODTEL          VARCHAR2(255);
    v_INTERFACE_NOME            VARCHAR2(255);
    v_MODULO_ID                 NUMBER(10, 0);
    v_PERFIL_ID                 NUMBER(10, 0);
    v_COUNT_PERMISSAO_INTERFACE NUMBER(10, 0) := 0;
    v_PERMISSAO_MENU_NEGADO_ID  NUMBER(10, 0);
    v_OPCAO_MENU_ID             NUMBER(10, 0);
    I                           NUMBER(10, 0) := 0;
    CURSOR CURSOR_INTERFACES IS -- BUSCA INTERFACES, MÓDULOS E PERFIS COM ACESSO AO MÓDULO
        SELECT DISTINCT FRM_CODFRM, FRM_TITULO, MOD_ID, PFM_PUSID FROM (
           SELECT FRM_CODFRM, FRM_TITULO, MOD_ID FROM GINTB_FORM FRM
                                                          LEFT JOIN GINTB_GRUPO_FORM GFR ON (FRM.FRM_GFRCODIGO = GFR.GFR_CODIGO)
                                                          LEFT JOIN EDUSOFT_SECURITY.TB_MODULO ON (FRM.FRM_MOD_DISCP like '%' || MOD_TIPOMOD || '%')
           WHERE FRM_TIPO IN ('DOCUM_TEMPLATE_HTML', 'DOCUM_TEMPLATE_ODT', 'CRUD')
             AND FRM_EXECSOMPROC = 1
       ) TABELA, EDUSOFT_SECURITY.TB_PERFIL_MODULO
        WHERE PFM_MODID = MOD_ID AND PFM_PUSID IS NOT NULL;
BEGIN

    dbms_output.put_line('[START] PROC_ADD_PERMISSAO_INTERFACE ');

    OPEN CURSOR_INTERFACES;
    FETCH CURSOR_INTERFACES INTO v_INTERFACE_CODTEL, v_INTERFACE_NOME, v_MODULO_ID, v_PERFIL_ID;
    WHILE CURSOR_INTERFACES%FOUND
        LOOP
            BEGIN
                I := I + 1;
                dbms_output.put_line('>>>>v_INTERFACE_CODTEL - ' || v_INTERFACE_CODTEL ||
                                     ' | v_INTERFACE_NOME - ' || v_INTERFACE_NOME ||
                                     ' | v_MODULO_ID - ' || TO_CHAR(v_MODULO_ID) ||
                                     ' | v_PERFIL_ID - ' || TO_CHAR(v_PERFIL_ID));

                -- BUSCA SE HÁ ALGUMA PERMISSÃO PARA A INTERFACE, NO MÓDULO PARA O PERFIL DE USUÁRIO
                BEGIN
                    SELECT COUNT(OME_ID)
                    INTO v_COUNT_PERMISSAO_INTERFACE
                    FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                             INNER JOIN EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                                ON (OME_ID = OMP_OMEID AND OMP_PUSID = v_PERFIL_ID)
                    WHERE OME_PREDEFID = v_INTERFACE_CODTEL
                        AND OME_TIPO = 'OPCAONOR'
                        AND OME_TIPOITEMPREDED = 'DYNAFORM'
                        AND OME_MODID = v_MODULO_ID
                        AND OMP_TIPPER != 'TPA1';
                EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                        v_COUNT_PERMISSAO_INTERFACE := 0;
                END;

                dbms_output.put_line('-----v_COUNT_PERMISSAO_INTERFACE - ' || TO_CHAR(v_COUNT_PERMISSAO_INTERFACE));

                IF v_COUNT_PERMISSAO_INTERFACE = 0 THEN -- CASO NÃO HAJA PERMISSÃO
                -- BUSCA SE HÁ ALGUMA PERMISSÃO NEGANDO O ACESSO AO MENU
                    BEGIN
                        SELECT OMP_ID
                        INTO v_PERMISSAO_MENU_NEGADO_ID
                        FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                 INNER JOIN EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                                            ON (OME_ID = OMP_OMEID AND OMP_PUSID = v_PERFIL_ID)
                        WHERE OME_PREDEFID = v_INTERFACE_CODTEL
                          AND OME_TIPO = 'OPCAONOR'
                          AND OME_TIPOITEMPREDED = 'DYNAFORM'
                          AND OME_MODID = v_MODULO_ID
                          AND OMP_TIPPER = 'TPA1'
                          AND ROWNUM = 1;
                    EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                            v_PERMISSAO_MENU_NEGADO_ID := 0;
                    END;

                    dbms_output.put_line('-----v_PERMISSAO_MENU_NEGADO_ID - ' ||
                                         TO_CHAR(COALESCE(v_PERMISSAO_MENU_NEGADO_ID, 0)));

                    IF v_PERMISSAO_MENU_NEGADO_ID != 0 THEN-- HÁ PERMISSÃO CADASTRADA CONTUDO ESTÁ NEGADA, ALTERARÁ PARA POSSUIR PERMISSÃO NA OPÇÃO DE MENU
                        dbms_output.put_line('-----UPDATE TB_OPCAO_MENU_PERMISSAO');

                        UPDATE EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO
                        SET OMP_TIPPER            = 'TPA3',
                            DATA_ULT_ALTERACAO    = current_timestamp,
                            USUARIO_ULT_ALTERACAO = 'EDUSOFT_PERMISSAO_INTERFACE',
                            VERSAO                = VERSAO + 1
                        WHERE OMP_ID = v_PERMISSAO_MENU_NEGADO_ID;

                        COMMIT;

                    ELSE -- SE NÃO HÁ PERMISSÃO CADASTRADA, VERIFICA/CRIA A OPÇÃO DE MENU PARA A INTERFACE E CRIA A PERMISSÃO PARA O PERFIL DE USUÁRIO
                        BEGIN
                            --                             VERIFICA SE HÁ OPÇÃO DE MENU CRIADA
                            BEGIN
                                SELECT OME_ID
                                INTO v_OPCAO_MENU_ID
                                FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                WHERE OME_PREDEFID = v_INTERFACE_CODTEL
                                  AND OME_TIPO = 'OPCAONOR'
                                  AND OME_TIPOITEMPREDED = 'DYNAFORM'
                                  AND OME_MODID = v_MODULO_ID
                                  AND ROWNUM = 1;
                            EXCEPTION
                                WHEN NO_DATA_FOUND THEN
                                    v_OPCAO_MENU_ID := 0;
                            END;

                            dbms_output.put_line('-----v_OPCAO_MENU_ID - ' || TO_CHAR(COALESCE(v_OPCAO_MENU_ID, 0)));

                            IF v_OPCAO_MENU_ID = 0 THEN-- CRIA A OPÇÃO DE MENU PARA A INTERFACE
                                dbms_output.put_line('-----INSERT TB_OPCAO_MENU');

                                INSERT INTO EDUSOFT_SECURITY.TB_OPCAO_MENU (OME_ID, OME_DESCRICAO, OME_ACTION, OME_MODID,
                                                                            OME_ORDEM, OME_TIPO, VERSAO,
                                                                            DATA_ULT_ALTERACAO,
                                                                            USUARIO_ULT_ALTERACAO, OME_TIPACEITAPER,
                                                                            OME_TIPOMENU, OME_TIPOACAOMENU,
                                                                            OME_TIPOITEMPREDED, OME_PREDEFID,
                                                                            OME_CODRELATORIO, OME_PARAMETROS,
                                                                            OME_EXIMOB)
                                VALUES (EDUSOFT_SECURITY.OME_ID.NEXTVAL, SUBSTR(v_INTERFACE_NOME, 0, 100) , 'jsf/geradorinterface/executa/interfaceexec.jsf?codigoForm=' || v_INTERFACE_CODTEL,
                                        v_MODULO_ID, 0, 'OPCAONOR', 0, current_timestamp, 'EDUSOFT_PERMISSAO_INTERFACE', 'TAP3', 'ITEMMENU', 'PREDEF', 'DYNAFORM', v_INTERFACE_CODTEL,
                                        '', '', 0);
                                COMMIT;

                                SELECT OME_ID
                                INTO v_OPCAO_MENU_ID
                                FROM EDUSOFT_SECURITY.TB_OPCAO_MENU
                                WHERE OME_PREDEFID = v_INTERFACE_CODTEL
                                  AND OME_TIPO = 'OPCAONOR'
                                  AND OME_TIPOITEMPREDED = 'DYNAFORM'
                                  AND OME_MODID = v_MODULO_ID
                                  AND ROWNUM = 1;

                                dbms_output.put_line('-----v_OPCAO_MENU_ID - ' || TO_CHAR(COALESCE(v_OPCAO_MENU_ID, 0)));
                            END IF;

                            dbms_output.put_line('-----[START] INSERT TB_OPCAO_MENU_PERMISSAO');

                            -- CRIA PERMISSÃO PARA O MENU
                            INSERT INTO EDUSOFT_SECURITY.TB_OPCAO_MENU_PERMISSAO (OMP_ID, OMP_PUSID, OMP_OMEID, OMP_TIPPER, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO)
                            VALUES (EDUSOFT_SECURITY.OMP_ID.NEXTVAL, v_PERFIL_ID , v_OPCAO_MENU_ID, 'TPA3', 0, current_timestamp, 'EDUSOFT_PERMISSAO_INTERFACE');

                            COMMIT;
                        END;
                    END IF;
                END IF;

                FETCH CURSOR_INTERFACES INTO v_INTERFACE_CODTEL, v_INTERFACE_NOME, v_MODULO_ID, v_PERFIL_ID;
            END;
        END LOOP;
    CLOSE CURSOR_INTERFACES;

    dbms_output.put_line('[END] PROC_ADD_PERMISSAO_INTERFACE');
END;
